"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Parser = void 0;
const TypeScriptCodeGenerator = __importStar(require("../TsGenerator"));
const Headers = __importStar(require("./components/Headers"));
const Parameters = __importStar(require("./components/Parameters"));
const RequestBodies = __importStar(require("./components/RequestBodies"));
const Responses = __importStar(require("./components/Responses"));
const Schemas = __importStar(require("./components/Schemas"));
const ConvertContext = __importStar(require("./ConverterContext"));
const Extractor = __importStar(require("./Extractor"));
const Paths = __importStar(require("./paths"));
const store_1 = require("./store");
const TypeNodeContext = __importStar(require("./TypeNodeContext"));
class Parser {
    constructor(entryPoint, rootSchema, noReferenceOpenApiSchema) {
        this.entryPoint = entryPoint;
        this.rootSchema = rootSchema;
        this.currentPoint = entryPoint;
        this.convertContext = ConvertContext.create();
        this.factory = TypeScriptCodeGenerator.Factory.create();
        this.store = store_1.Store.create(this.factory, noReferenceOpenApiSchema);
        this.initialize();
    }
    initialize() {
        const toTypeNodeContext = TypeNodeContext.create(this.entryPoint, this.store, this.factory, this.convertContext);
        const rootSchema = this.rootSchema;
        if (rootSchema.components) {
            if (rootSchema.components.schemas) {
                Schemas.generateNamespace(this.entryPoint, this.currentPoint, this.store, this.factory, rootSchema.components.schemas, toTypeNodeContext, this.convertContext);
            }
            if (rootSchema.components.headers) {
                Headers.generateNamespace(this.entryPoint, this.currentPoint, this.store, this.factory, rootSchema.components.headers, toTypeNodeContext, this.convertContext);
            }
            if (rootSchema.components.responses) {
                Responses.generateNamespace(this.entryPoint, this.currentPoint, this.store, this.factory, rootSchema.components.responses, toTypeNodeContext, this.convertContext);
            }
            if (rootSchema.components.parameters) {
                Parameters.generateNamespace(this.entryPoint, this.currentPoint, this.store, this.factory, rootSchema.components.parameters, toTypeNodeContext, this.convertContext);
            }
            if (rootSchema.components.requestBodies) {
                RequestBodies.generateNamespace(this.entryPoint, this.currentPoint, this.store, this.factory, rootSchema.components.requestBodies, toTypeNodeContext, this.convertContext);
            }
            // if (rootSchema.components.securitySchemes) {
            //   SecuritySchemas.generateNamespace(this.entryPoint, currentPoint, store, factory, rootSchema.components.securitySchemes);
            // }
            // if (rootSchema.components.pathItems) {
            //   PathItems.generateNamespace(this.entryPoint, currentPoint, store, factory, rootSchema.components.pathItems, toTypeNodeContext);
            // }
            // TODO Feature Development
            // if (rootSchema.components.links) {
            //   statements.push(Links.generateNamespace(this.entryPoint, currentPoint, factory, rootSchema.components.links));
            // }
            // TODO Feature Development
            // if (rootSchema.components.callbacks) {
            //   statements.push(Callbacks.generateNamespace(this.entryPoint, currentPoint, factory, rootSchema.components.callbacks));
            // }
        }
        if (rootSchema.paths) {
            Paths.generateStatements(this.entryPoint, this.currentPoint, this.store, this.factory, rootSchema.paths, toTypeNodeContext, this.convertContext);
        }
    }
    getCodeGeneratorParamsArray(allowOperationIds) {
        return Extractor.generateCodeGeneratorParamsArray(this.store, this.convertContext, allowOperationIds);
    }
    getOpenApiTypeDefinitionStatements() {
        return this.store.getRootStatements();
    }
    getAdditionalTypeStatements() {
        return this.store.getAdditionalStatements();
    }
}
exports.Parser = Parser;
//# sourceMappingURL=Parser.js.map